@@ -47,7 +47,9 @@ def pre_compute(expr, data, comfortable_memory=None, chunksize=2**18, **kwargs):
     leaf = oexpr._leaves()[0]
     pth = list(path(oexpr, leaf))
     if len(pth) >= 2 and isinstance(pth[-2], (Projection, Field)):
-        kwargs['usecols'] = pth[-2].fields
+        # NOTE: FIXME: We pass the column names through `str` to workaround a
+        # PY2 Pandas bug with strings / unicode objects.
+        kwargs['usecols'] = list(map(str, pth[-2].fields))
 
     if chunksize:
         return into(chunks(pd.DataFrame), data, dshape=leaf.dshape, **kwargs)
@@ -72,7 +74,7 @@ def pre_compute(expr, data, **kwargs):
 
 
 def compute_chunk(chunk, chunk_expr, part):
-    return compute(chunk_expr, {chunk: part})
+    return compute(chunk_expr, {chunk: part}, return_type='native')
 
 
 @dispatch(Expr, pandas.io.parsers.TextFileReader)
@@ -92,4 +94,4 @@ def compute_down(expr, data, map=None, **kwargs):
     elif isinstance(parts[0], (Iterable, Iterator)):
         intermediate = concat(parts)
 
-    return compute(agg_expr, {agg: intermediate})
+    return compute(agg_expr, {agg: intermediate}, return_type='native')
