@@ -49,21 +49,57 @@ except ImportError:
     DaskDataFrame = pd.DataFrame
     DaskSeries = pd.Series
 
-from ..dispatch import dispatch
-
 from .core import compute, compute_up, base
-
-from ..expr import (Projection, Field, Sort, Head, Tail, Broadcast, Selection,
-                    Reduction, Distinct, Join, By, Summary, Label, ReLabel,
-                    Map, Apply, Merge, std, var, Like, Slice, summary,
-                    ElemWise, DateTime, Millisecond, Expr, Symbol, IsIn,
-                    UTCFromTimestamp, nelements, DateTimeTruncate, count,
-                    UnaryStringFunction, nunique, Coerce, Concat,
-                    isnan, notnull, Shift)
-from ..expr import UnaryOp, BinOp, Interp
-from ..expr import symbol, common_subexpression
-
 from ..compatibility import _inttypes
+from ..dispatch import dispatch
+from ..expr import (
+    Apply,
+    BinOp,
+    Broadcast,
+    By,
+    Coalesce,
+    Coerce,
+    Concat,
+    DateTime,
+    DateTimeTruncate,
+    Distinct,
+    ElemWise,
+    Expr,
+    Field,
+    Head,
+    Interp,
+    IsIn,
+    Join,
+    Label,
+    Like,
+    Map,
+    Merge,
+    Millisecond,
+    Projection,
+    ReLabel,
+    Reduction,
+    Sample,
+    Selection,
+    Shift,
+    Slice,
+    Sort,
+    Summary,
+    Tail,
+    UTCFromTimestamp,
+    UnaryOp,
+    UnaryStringFunction,
+    common_subexpression,
+    count,
+    isnan,
+    nelements,
+    notnull,
+    nunique,
+    std,
+    summary,
+    symbol,
+    var,
+    StrCat,
+)
 
 __all__ = []
 
@@ -91,8 +127,7 @@ def compute_up(t, data, **kwargs):
 
 @dispatch(Broadcast, (DataFrame, DaskDataFrame))
 def compute_up(t, df, **kwargs):
-    d = dict((t._child[c]._expr, df[c]) for c in t._child.fields)
-    return compute(t._expr, d)
+    return compute(t._full_expr, df, return_type='native')
 
 
 @dispatch(Broadcast, Series)
@@ -122,14 +157,9 @@ def compute_up(t, data, **kwargs):
     else:
         return t.op(t.lhs, data)
 
-
-@dispatch(BinOp, (Series, DaskSeries), (Series, base, DaskSeries))
-def compute_up(t, lhs, rhs, **kwargs):
-    return t.op(lhs, rhs)
-
-
-@dispatch(BinOp, (Series, base, DaskSeries), (Series, DaskSeries))
-def compute_up(t, lhs, rhs, **kwargs):
+@compute_up.register(BinOp, (Series, DaskSeries), (Series, base, DaskSeries))
+@compute_up.register(BinOp, (Series, base, DaskSeries), (Series, DaskSeries))
+def compute_up_binop(t, lhs, rhs, **kwargs):
     return t.op(lhs, rhs)
 
 
@@ -147,7 +177,7 @@ def compute_up(expr, df, **kwargs):
     return compute_up(
         expr,
         df,
-        compute(expr.predicate, {expr._child: df}),
+        compute(expr.predicate, {expr._child: df}, return_type='native'),
         **kwargs
     )
 
@@ -255,9 +285,10 @@ def compute_up(expr, data, **kwargs):
     return compute_up(expr._child.distinct().count(), data, **kwargs)
 
 
-string_func_names = {
-    'strlen': 'len',
-}
+string_func_names = {'str_len': 'len',
+                     'strlen': 'len',
+                     'str_upper': 'upper',
+                     'str_lower': 'lower'}
 
 
 @dispatch(UnaryStringFunction, Series)
@@ -266,6 +297,12 @@ def compute_up(expr, data, **kwargs):
     return getattr(data.str, string_func_names.get(name, name))()
 
 
+@dispatch(StrCat, Series, Series)
+def compute_up(expr, lhs_data, rhs_data, **kwargs):
+    res = lhs_data.str.cat(rhs_data, sep=expr.sep)
+    return res
+
+
 def unpack(seq):
     """ Unpack sequence of length one
 
@@ -291,7 +328,7 @@ def get_grouper(c, grouper, df):
 
 @dispatch(By, Expr, NDFrame)
 def get_grouper(c, grouper, df):
-    g = compute(grouper, {c._child: df})
+    g = compute(grouper, {c._child: df}, return_type='native')
     if isinstance(g, Series):
         return g
     if isinstance(g, DataFrame):
@@ -306,7 +343,7 @@ def get_grouper(c, grouper, df):
 @dispatch(By, Reduction, Grouper, NDFrame)
 def compute_by(t, r, g, df):
     names = [r._name]
-    preapply = compute(r._child, {t._child: df})
+    preapply = compute(r._child, {t._child: df}, return_type='native')
 
     # Pandas and Blaze column naming schemes differ
     # Coerce DataFrame column names to match Blaze's names
@@ -320,7 +357,7 @@ def compute_by(t, r, g, df):
     gb = group_df.groupby(g)
     groups = gb[names[0] if isscalar(t.apply._child.dshape.measure) else names]
 
-    return compute_up(r, groups)  # do reduction
+    return compute_up(r, groups, return_type='native')  # do reduction
 
 
 name_dict = dict()
@@ -395,8 +432,8 @@ def compute_by(t, s, g, df):
 
     preapply = DataFrame(dict(
         zip([name for name, _ in is_field[True]],
-            [compute(col._child, {t._child: df}) for _, col in is_field[True]]
-            )
+            [compute(col._child, {t._child: df}, return_type='native')
+             for _, col in is_field[True]])
         )
     )
 
@@ -416,7 +453,10 @@ def compute_by(t, s, g, df):
     result = groups.agg(d)
 
     scope = dict((v, result[k]) for k, v in two.items())
-    cols = [compute(expr.label(name), scope) for name, expr in three.items()]
+    cols = [
+        compute(expr.label(name), scope, return_type='native')
+        for name, expr in three.items()
+    ]
 
     result2 = pd.concat(cols, axis=1)
 
@@ -514,6 +554,34 @@ def compute_up(t, s, **kwargs):
         return s.order(ascending=t.ascending)
 
 
+@dispatch(Sample, (Series, DataFrame))
+def compute_up(t, df, **kwargs):
+    from math import modf
+    if t.frac is not None:
+        # Work around annoying edge case: Python's round() builtin (which
+        # Pandas' sample() uses) rounds 0.5, 2.5, 4.5, ... down to 0, 2, 4, ...,
+        # while it rounds 1.5, 3.5, 5.5, ... up.  This is inconsistent with any
+        # sane implementation of floating point rounding, including SQL's, so
+        # we work around it here.
+        fractional, integral = modf(t.frac * df.shape[0])
+        n = int(integral + (0 if fractional < 0.5 else 1))
+    else:
+        n = min(t.n, df.shape[0])
+    return df.sample(n=n)
+
+
+@dispatch(Sample, (DaskDataFrame, DaskSeries))
+def compute_up(t, df, **kwargs):
+    # Dask doesn't support sample(n=...), only sample(frac=...), so we have a
+    # separate dispatch for dask objects.
+    if t.frac is not None:
+        frac = t.frac
+    else:
+        nrows = len(df)
+        frac = float(min(t.n, nrows)) / nrows
+    return df.sample(frac=frac)
+
+
 @dispatch(Head, (Series, DataFrame, DaskDataFrame, DaskSeries))
 def compute_up(t, df, **kwargs):
     return df.head(t.n)
@@ -574,13 +642,19 @@ def compute_up(t, df, **kwargs):
 def compute_up(t, df, scope=None, **kwargs):
     subexpression = common_subexpression(*t.children)
     scope = merge_dicts(scope or {}, {subexpression: df})
-    children = [compute(_child, scope) for _child in t.children]
+    children = [
+        compute(_child, scope, return_type='native')
+        for _child in t.children
+    ]
     return pd.concat(children, axis=1)
 
 
 @dispatch(Summary, (DataFrame, DaskDataFrame))
 def compute_up(expr, data, **kwargs):
-    values = [compute(val, {expr._child: data}) for val in expr.values]
+    values = [
+        compute(val, {expr._child: data}, return_type='native')
+        for val in expr.values
+    ]
     if expr.keepdims:
         return type(data)([values], columns=expr.fields)
     else:
@@ -589,7 +663,10 @@ def compute_up(expr, data, **kwargs):
 
 @dispatch(Summary, (Series, DaskSeries))
 def compute_up(expr, data, **kwargs):
-    result = tuple(compute(val, {expr._child: data}) for val in expr.values)
+    result = tuple(
+        compute(val, {expr._child: data}, return_type='native')
+        for val in expr.values
+    )
     if expr.keepdims:
         result = [result]
     return result
@@ -682,3 +759,36 @@ def compute_up(expr, data, **kwargs):
 @dispatch(Shift, Series)
 def compute_up(expr, data, **kwargs):
     return data.shift(expr.n)
+
+
+def array_coalesce(expr, lhs, rhs, wrap=None, **kwargs):
+    res = np.where(pd.isnull(lhs), rhs, lhs)
+    if not expr.dshape.shape:
+        res = res.item()
+    elif wrap:
+        res = wrap(res)
+    return res
+
+
+@compute_up.register(
+    Coalesce, (Series, DaskSeries), (np.ndarray, Series, base, DaskSeries)
+)
+@compute_up.register(
+    Coalesce,
+    (Series, base, DaskSeries), (np.ndarray, Series, DaskSeries)
+)
+def compute_up_coalesce(expr, lhs, rhs, **kwargs):
+    return array_coalesce(expr, lhs, rhs, type(lhs))
+
+
+
+@dispatch(Coalesce, (Series, DaskSeries, base))
+def compute_up(t, data, **kwargs):
+    if isinstance(t.lhs, Expr):
+        lhs = data
+        rhs = t.rhs
+    else:
+        lhs = t.lhs
+        rhs = data
+
+    return compute_up_coalesce(t, lhs, rhs)
