@@ -1,10 +1,12 @@
 from __future__ import absolute_import, division, print_function
 
-default_map = [map]
+default_map = map
+
 
 def set_default_pmap(func):
-    default_map[0] = func
+    global default_map
+    default_map = func
 
 
 def get_default_pmap():
-    return default_map[0]
+    return default_map
