@@ -12,9 +12,11 @@ from blaze.expr import by, symbol, Expr, Symbol
 from blaze.dispatch import dispatch
 from blaze.compatibility import raises, reduce
 from blaze.utils import example
+from blaze.interactive import into
 
 import pandas as pd
 import numpy as np
+import dask.array as da
 
 
 def test_errors():
@@ -88,8 +90,8 @@ def test_top_then_bottom_then_top_again_etc():
 
 def test_swap_resources_into_scope():
 
-    from blaze import Data
-    t = Data([1, 2, 3], dshape='3 * int', name='t')
+    from blaze import data
+    t = data([1, 2, 3], dshape='3 * int', name='t')
     expr, scope = swap_resources_into_scope(t.head(2), {t: t.data})
 
     assert t._resources()
@@ -110,7 +112,7 @@ def test_compute_up_on_dict():
 
 def test_pre_compute_on_multiple_datasets_is_selective():
     from odo import CSV
-    from blaze import Data
+    from blaze import data
     from blaze.cached import CachedDataset
 
     df = pd.DataFrame([[1, 'Alice',   100],
@@ -121,7 +123,7 @@ def test_pre_compute_on_multiple_datasets_is_selective():
     iris = CSV(example('iris.csv'))
     dset = CachedDataset({'df': df, 'iris': iris})
 
-    d = Data(dset)
+    d = data(dset)
     assert str(compute(d.df.amount)) == str(df.amount)
 
 
@@ -145,3 +147,16 @@ def test_simple_add(n):
     x = symbol('x', 'int')
     expr = reduce(operator.add, [x] * n)
     assert compute(expr, 1) == n
+
+
+@pytest.mark.parametrize('data,expr,ret_type,exp_type', [
+    (1, symbol('x', 'int'), 'native', int),
+    (1, symbol('x', 'int'), 'core', int),
+    # use dask array to test core since isn't core type
+    (into(da.core.Array, [1, 2], chunks=(10,)), symbol('x', '2 * int'), 'core', pd.Series),  # test 1-d to series
+    (into(da.core.Array, [{'a': 1, 'b': 2}, {'a': 3, 'b': 4}], chunks=(10,10)), symbol('x', '2 * {a: int, b: int}'), 'core', pd.DataFrame),  # test 2-d tabular to dataframe
+    (into(da.core.Array, [[1, 2], [3, 4]], chunks=(10, 10)), symbol('x', '2 *  2 * int'), 'core', np.ndarray),  # test 2-d non tabular to ndarray
+    ([1, 2], symbol('x', '2 * int') , tuple, tuple)
+])
+def test_compute_return_type(data, expr, ret_type, exp_type):
+    assert isinstance(compute(expr, data, return_type=ret_type), exp_type)
