@@ -14,10 +14,10 @@ from string import ascii_lowercase
 
 from blaze.compute.core import compute
 from blaze.compute.pandas import pdsort
-from blaze import dshape, discover, transform
+from blaze import dshape, discover, transform, broadcast_collect
 from blaze.expr import symbol, join, by, summary, distinct, shape
 from blaze.expr import (merge, exp, mean, count, nunique, sum, min, max, any,
-                        var, std, concat)
+                        var, std, concat, coalesce)
 from blaze.compatibility import builtins, xfail, assert_series_equal
 
 
@@ -45,6 +45,50 @@ dfbig = DataFrame([['Alice', 'F', 100, 1],
                   columns=['name', 'sex', 'amount', 'id'])
 
 
+# for now jsut copy this, but will open a PR to see if we can remove some of
+# the repetitive copying
+tbgr = symbol('tbgr',
+              """ var * {name: string,
+                         sex: string[1],
+                         amount: int,
+                         id: int,
+                         comment: ?string}
+              """)
+
+dfbgr = DataFrame([['Alice', 'F', 100, 1, 'Alice comment'],
+                   ['Alice', 'F', 100, 3, None],
+                   ['Drew', 'F', 100, 4, 'Drew comment'],
+                   ['Drew', 'M', 100, 5, 'Drew comment 2'],
+                   ['Drew', 'M', 200, 5, None]],
+                  columns=['name', 'sex', 'amount', 'id', 'comment'])
+
+
+@pytest.fixture(scope='module')
+def df_add_null():
+    rows = [(None, 'M', 300, 6),
+            ('first', None, 300, 6),
+            (None, None, 300, 6)]
+    df_add_null = dfbig.append(DataFrame(rows,
+                                         columns=dfbig.columns
+                                         ), ignore_index=True)
+
+    return df_add_null
+
+
+def test_series_broadcast():
+    s = Series([1, 2, 3], name='a')
+    t = symbol('t', 'var * {a: int64}')
+    bcast = broadcast_collect(expr=(2 * t.a - t.a**2))
+    result = compute(bcast, s)
+    assert_series_equal(result, 2 * s - s**2)
+
+
+def test_frame_broadcast():
+    bcast = broadcast_collect(expr=t.amount * t.id)
+    result = compute(bcast, df)
+    assert_series_equal(result, df.amount * df.id)
+
+
 def test_series_columnwise():
     s = Series([1, 2, 3], name='a')
     t = symbol('t', 'var * {a: int64}')
@@ -369,6 +413,22 @@ def test_tail():
     tm.assert_frame_equal(compute(t.tail(1), df), df.tail(1))
 
 
+def test_sample():
+    samp = compute(t.sample(n=2), df)
+    assert len(samp) == 2
+
+
+def test_sample_frac_rounding_edge_case():
+    samp_big = compute(tbig.sample(frac=0.1), dfbig)
+    assert len(samp_big) == int(np.ceil(len(dfbig) * 0.1))
+
+
+def test_sample_clip():
+    samp_series = compute(t.name.sample(n=2*len(df)), df)
+    samp_df = compute(t.sample(n=2*len(df)), df)
+    assert len(samp_series) == len(samp_df) == len(df)
+
+
 def test_label():
     expected = df['amount'] * 10
     expected.name = 'foo'
@@ -629,13 +689,55 @@ def test_like():
     tm.assert_frame_equal(result, expected)
 
 
-def test_strlen():
-    expr = t.name.strlen()
+def test_str_len():
+    expr = t.name.str_len()
     expected = pd.Series([5, 3, 5], name='name')
     result = compute(expr, df).reset_index(drop=True)
     assert_series_equal(expected, result)
 
 
+def test_str_upper():
+    expr = t.name.str_upper()
+    expected = pd.Series(['ALICE', 'BOB', 'ALICE'], name='name')
+    result = compute(expr, df).reset_index(drop=True)
+    assert_series_equal(expected, result)
+
+
+def test_str_lower():
+    expr = t.name.str_lower()
+    expected = pd.Series(['alice', 'bob', 'alice'], name='name')
+    result = compute(expr, df).reset_index(drop=True)
+    assert_series_equal(expected, result)
+
+
+def test_str_cat():
+    res = compute(tbig.name.str_cat(tbig.sex), dfbig)
+    assert all(dfbig.name.str.cat(dfbig.sex) == res)
+
+
+def test_str_cat_sep():
+    res = compute(tbig.name.str_cat(tbig.sex, sep=' -- '), dfbig)
+    assert all(dfbig.name.str.cat(dfbig.sex, sep=' -- ') == res)
+
+
+def test_str_cat_null_row(df_add_null):
+    res = compute(tbig.name.str_cat(tbig.sex, sep=' -- '), df_add_null)
+    exp_res = df_add_null.name.str.cat(df_add_null.sex, sep=' -- ')
+
+    assert all(exp_res.isnull() == res.isnull())
+    assert all(exp_res[~exp_res.isnull()] == res[~res.isnull()])
+
+
+def test_str_cat_chain_operation():
+    expr = tbgr.name.str_cat(tbgr.comment.str_cat(tbgr.sex, sep=' --- '),
+                             sep=' +++ ')
+    res = compute(expr, dfbgr)
+    exp_res = dfbgr.name.str.cat(dfbgr.comment.str.cat(dfbgr.sex, sep=' --- '),
+                                 sep=' +++ ')
+    assert all(exp_res.isnull() == res.isnull())
+    assert all(exp_res[~exp_res.isnull()] == res[~res.isnull()])
+
+
 def test_rowwise_by():
     f = lambda _, id, name: id + len(name)
     expr = by(t.map(f, 'int'), total=t.amount.sum())
@@ -890,3 +992,38 @@ def test_selection_inner_inputs():
 def test_by_with_reduction_on_df():
     expr = by(tbig.name, id_sum=tbig.id.sum(), count=tbig.count())
     compute(expr, dfbig)
+
+
+def test_coalesce():
+    data = pd.Series([0, None, 1, None, 2, None], dtype=object)
+
+    s = symbol('s', 'var * ?int')
+    t = symbol('t', 'int')
+    u = symbol('u', '?int')
+    v = symbol('v', 'var * int')
+    w = symbol('w', 'var * ?int')
+
+    # array to scalar
+    tm.assert_series_equal(
+        compute(coalesce(s, t), {s: data, t: -1}),
+        pd.Series([0, -1, 1, -1, 2, -1], dtype=object),
+    )
+    # array to scalar with NULL
+    tm.assert_series_equal(
+        compute(coalesce(s, u), {s: data, u: None}),
+        pd.Series([0, None, 1, None, 2, None], dtype=object),
+    )
+    # array to array
+    tm.assert_series_equal(
+        compute(coalesce(s, v), {
+            s: data, v: np.array([-1, -2, -3, -4, -5, -6]),
+        }),
+        pd.Series([0, -2, 1, -4, 2, -6], dtype=object),
+    )
+    # array to array with NULL
+    tm.assert_series_equal(
+        compute(coalesce(s, w), {
+            s: data, w: np.array([-1, None, -3, -4, -5, -6]),
+        }),
+        pd.Series([0, None, 1, -4, 2, -6], dtype=object),
+    )
