@@ -1,15 +1,15 @@
 from __future__ import absolute_import, division, print_function
 
+from collections import Mapping
+import datetime
 import numbers
 import inspect
 
 from pprint import pformat
 from functools import reduce, partial
 
-import numpy as np
 import toolz
 from toolz import unique, concat, first
-import pandas as pd
 
 from ..compatibility import _strtypes
 from ..dispatch import dispatch
@@ -18,7 +18,7 @@ from ..utils import ordered_intersect
 __all__ = ['Node', 'path', 'common_subexpression', 'eval_str']
 
 
-base = (numbers.Number,) + _strtypes
+base = (numbers.Number,) + _strtypes + (datetime.datetime, datetime.timedelta)
 
 
 def isidentical(a, b):
@@ -118,13 +118,13 @@ class Node(object):
 
         >>> t = symbol('t', 'var * {id: int32, name: string}')
         >>> t._leaves()
-        [t]
+        [<`t` symbol; dshape='var * {id: int32, name: string}'>]
         >>> by(t.name, count=t.id.nunique())._leaves()
-        [t]
+        [<`t` symbol; dshape='var * {id: int32, name: string}'>]
 
         >>> v = symbol('v', 'var * {id: int32, city: string}')
-        >>> join(t, v)._leaves()
-        [t, v]
+        >>> join(t, v)._leaves() == [t, v]
+        True
         """
 
         if not self._inputs:
@@ -148,9 +148,6 @@ class Node(object):
         ]
         return '%s(%s)' % (type(self).__name__, ', '.join(rep))
 
-    def __repr__(self):
-        return str(self)
-
     def _traverse(self):
         """ Traverse over tree, yielding all subtrees and leaves """
         yield self
@@ -367,12 +364,12 @@ def subs(o, d):
     return _subs(o, d)
 
 
-@dispatch((tuple, list), dict)
+@dispatch((tuple, list), Mapping)
 def _subs(o, d):
     return type(o)([subs(arg, d) for arg in o])
 
 
-@dispatch(Node, dict)
+@dispatch(Node, Mapping)
 def _subs(o, d):
     """
 
@@ -385,7 +382,7 @@ def _subs(o, d):
     return type(o)(*newargs)
 
 
-@dispatch(object, dict)
+@dispatch(object, Mapping)
 def _subs(o, d):
     """ Private dispatched version of ``subs``
 
@@ -402,7 +399,7 @@ def path(a, b):
     >>> t = symbol('t', 'var * {name: string, amount: int, id: int}')
     >>> expr = t.amount.sum()
     >>> list(path(expr, t))
-    [sum(t.amount), t.amount, t]
+    [sum(t.amount), t.amount, <`t` symbol; dshape='...'>]
     """
     while not a.isidentical(b):
         yield a
@@ -424,7 +421,7 @@ def common_subexpression(expr, *exprs):
     >>> from blaze.expr import symbol
     >>> t = symbol('t', 'var * {x: int, y: int}')
     >>> common_subexpression(t.x, t.y)
-    t
+    <`t` symbol; dshape='var * {x: int32, y: int32}'>
     """
     # only one expression has itself as a common subexpression
     if not exprs:
