@@ -1,10 +1,21 @@
 from __future__ import absolute_import, division, print_function
 
+from collections import Mapping
 from keyword import iskeyword
 import re
 
 import datashape
-from datashape import dshape, DataShape, Record, Var, Mono, Fixed
+from datashape import (
+    dshape,
+    DataShape,
+    Record,
+    Var,
+    Mono,
+    Fixed,
+    promote,
+    Option,
+    Null,
+)
 from datashape.predicates import isscalar, iscollection, isboolean, isrecord
 import numpy as np
 from odo.utils import copydoc
@@ -16,12 +27,14 @@ from ..compatibility import _strtypes, builtins, boundmethod, PY2
 from .core import Node, subs, common_subexpression, path
 from .method_dispatch import select_functions
 from ..dispatch import dispatch
-from .utils import hashable_index, replace_slices
+from .utils import hashable_index, replace_slices, maxshape
 from ..utils import attribute
 
 
 __all__ = [
     'Apply',
+    'Cast',
+    'Coalesce',
     'Coerce',
     'ElemWise',
     'Expr',
@@ -35,10 +48,11 @@ __all__ = [
     'Slice',
     'Symbol',
     'apply',
+    'cast',
+    'coalesce',
     'coerce',
     'discover',
     'label',
-    'label',
     'ndim',
     'projection',
     'relabel',
@@ -157,13 +171,16 @@ class Expr(Node):
 
     @property
     def fields(self):
-        if isinstance(self.dshape.measure, Record):
-            return self.dshape.measure.names
-        elif isinstance(self.dshape.measure, datashape.Map):
+        measure = self.dshape.measure
+        if isinstance(self.dshape.measure, Option):
+            measure = measure.ty
+        if isinstance(measure, Record):
+            return measure.names
+        elif isinstance(measure, datashape.Map):
             if not isrecord(self.dshape.measure.value):
                 raise TypeError('Foreign key must reference a '
                                 'Record datashape')
-            return self.dshape.measure.value.names
+            return measure.value.names
         name = getattr(self, '_name', None)
         if name is not None:
             return [self._name]
@@ -274,6 +291,13 @@ def _symbol_key(args, kwargs):
     return (name, ds, token)
 
 
+def sanitized_dshape(dshape, width=50):
+    pretty_dshape = datashape.pprint(dshape, width=width).replace('\n', '')
+    if len(pretty_dshape) > width:
+        pretty_dshape = "{}...".format(pretty_dshape[:width])
+    return pretty_dshape
+
+
 class Symbol(Expr):
     """
     Symbolic data.  The leaf of a Blaze expression
@@ -282,7 +306,7 @@ class Symbol(Expr):
     --------
     >>> points = symbol('points', '5 * 3 * {x: int, y: int}')
     >>> points
-    points
+    <`points` symbol; dshape='5 * 3 * {x: int32, y: int32}'>
     >>> points.dshape
     dshape("5 * 3 * {x: int32, y: int32}")
     """
@@ -299,6 +323,10 @@ class Symbol(Expr):
         self._token = token
         self._hash = None
 
+    def __repr__(self):
+        fmt = "<`{}` symbol; dshape='{}'>"
+        return fmt.format(self._name, sanitized_dshape(self.dshape))
+
     def __str__(self):
         return self._name or ''
 
@@ -312,7 +340,7 @@ def symbol(name, dshape, token=None):
     return Symbol(name, dshape, token=token or 0)
 
 
-@dispatch(Symbol, dict)
+@dispatch(Symbol, Mapping)
 def _subs(o, d):
     """ Subs symbols using symbol function
 
@@ -643,9 +671,8 @@ class ReLabel(ElemWise):
 
 @copydoc(ReLabel)
 def relabel(child, labels=None, **kwargs):
-    labels = labels or dict()
-    labels = toolz.merge(labels, kwargs)
-    labels = dict((k, v) for k, v in labels.items() if k != v)
+    labels = {k: v
+              for k, v in toolz.merge(labels or {}, kwargs).items() if k != v}
     label_keys = set(labels)
     fields = child.fields
     if not label_keys.issubset(fields):
@@ -654,7 +681,7 @@ def relabel(child, labels=None, **kwargs):
                          ', '.join(map(repr, non_existent_fields)))
     if not labels:
         return child
-    if isinstance(labels, dict):  # Turn dict into tuples
+    if isinstance(labels, Mapping):  # Turn dict into tuples
         labels = tuple(sorted(labels.items()))
     if isscalar(child.dshape.measure):
         if child._name == labels[0][0]:
@@ -790,6 +817,118 @@ def coerce(expr, to):
     return Coerce(expr, dshape(to) if isinstance(to, _strtypes) else to)
 
 
+class Cast(Expr):
+    """Cast an expression to a different type.
+
+    This is only an expression time operation.
+
+    Examples
+    --------
+    >>> s = symbol('s', '?int64')
+    >>> s.cast('?int32').dshape
+    dshape("?int32")
+
+    # Cast to correct mislabeled optionals
+    >>> s.cast('int64').dshape
+    dshape("int64")
+
+    # Cast to give concrete dimension length
+    >>> t = symbol('t', 'var * float32')
+    >>> t.cast('10 * float32').dshape
+    dshape("10 * float32")
+    """
+    __slots__ = '_hash', '_child', 'to'
+
+    def _dshape(self):
+        return self.to
+
+    def __str__(self):
+        return 'cast(%s, to=%r)' % (self._child, str(self.to))
+
+
+@copydoc(Cast)
+def cast(expr, to):
+    return Cast(expr, dshape(to) if isinstance(to, _strtypes) else to)
+
+
+Expr.cast = cast  # method of all exprs
+
+
+def binop_name(expr):
+    if not isscalar(expr.dshape.measure):
+        return None
+    l = getattr(expr.lhs, '_name', None)
+    r = getattr(expr.rhs, '_name', None)
+    if bool(l) ^ bool(r):
+        return l or r
+    elif l == r:
+        return l
+
+    return None
+
+
+def binop_inputs(expr):
+    if isinstance(expr.lhs, Expr):
+        yield expr.lhs
+    if isinstance(expr.rhs, Expr):
+        yield expr.rhs
+
+
+class Coalesce(Expr):
+    """SQL like coalesce.
+
+    coalesce(a, b) = {
+        a if a is not NULL
+        b otherwise
+    }
+
+    Examples
+    --------
+    >>> coalesce(1, 2)
+    1
+    >>> coalesce(1, None)
+    1
+    >>> coalesce(None, 2)
+    2
+    >>> coalesce(None, None) is None
+    True
+    """
+    __slots__ = '_hash', 'lhs', 'rhs', 'dshape'
+    __inputs__ = 'lhs', 'rhs'
+
+    def __str__(self):
+        return 'coalesce(%s, %s)' % (self.lhs, self.rhs)
+
+    _name = property(binop_name)
+
+    @property
+    def _inputs(self):
+        return tuple(binop_inputs(self))
+
+
+@copydoc(Coalesce)
+def coalesce(a, b):
+    a_dshape = discover(a)
+    a_measure = a_dshape.measure
+    isoption = isinstance(a_measure, Option)
+    if isoption:
+        a_measure = a_measure.ty
+    isnull = isinstance(a_measure, Null)
+    if isnull:
+        # a is always null, this is just b
+        return b
+
+    if not isoption:
+        # a is not an option, this is just a
+        return a
+
+    b_dshape = discover(b)
+    return Coalesce(a, b, DataShape(*(
+        maxshape((a_dshape.shape, b_dshape.shape)) +
+        (promote(a_measure, b_dshape.measure),)
+    )))
+
+
 dshape_method_list = list()
 schema_method_list = list()
 method_properties = set()
@@ -845,6 +984,7 @@ dshape_method_list.extend([
 schema_method_list.extend([
     (isscalar, set([label, relabel, coerce])),
     (isrecord, set([relabel])),
+    (lambda ds: isinstance(ds, Option), {coalesce}),
 ])
 
 method_properties.update([shape, ndim])
