@@ -8,6 +8,7 @@ from flask.testing import FlaskClient
 from odo import resource
 import requests
 from requests.packages.urllib3.exceptions import InsecureRequestWarning
+from toolz import assoc
 
 from ..expr import Expr
 from ..dispatch import dispatch
@@ -31,7 +32,7 @@ def content(response):
 
 def ok(response):
     if isinstance(response, flask.Response):
-        return 'OK' in response.status
+        return 200 <= response.status_code <= 299
     if isinstance(response, requests.Response):
         return response.ok
 
@@ -94,9 +95,9 @@ class Client(object):
     --------
 
     >>> # This example matches with the docstring of ``Server``
-    >>> from blaze import Data
+    >>> from blaze import data
     >>> c = Client('localhost:6363')
-    >>> t = Data(c) # doctest: +SKIP
+    >>> t = data(c) # doctest: +SKIP
 
     See Also
     --------
@@ -123,6 +124,48 @@ class Client(object):
 
         return dshape(content(response).decode('utf-8'))
 
+    def add(self, name, resource_uri, *args, **kwargs):
+        """Add the given resource URI to the Blaze server.
+
+        Parameters
+        ----------
+        name : str
+            The name to give the resource
+        resource_uri : str
+            The URI string describing the resource to add to the server, e.g
+            'sqlite:///path/to/file.db::table'
+        imports : list
+            A list of string names for any modules that must be imported on
+            the Blaze server before the resource can be added. This is identical
+            to the `imports` field in a Blaze server YAML file.
+        args : any, optional
+            Any additional positional arguments that can be passed to the
+            ``blaze.resource`` constructor for this resource type
+        kwargs : any, optional
+            Any additional keyword arguments that can be passed to the
+            ``blaze.resource`` constructor for this resource type
+        """
+        payload = {name: {'source': resource_uri}}
+        imports = kwargs.pop('imports', None)
+        if imports is not None:
+            payload[name]['imports'] = imports
+        if args:
+            payload[name]['args'] = args
+        if kwargs:
+            payload[name]['kwargs'] = kwargs
+
+        response = post(self, '/add', auth=self.auth,
+                        data=self.serial.dumps(payload),
+                        headers={'Content-Type': 'application/vnd.blaze+' + self.serial.name})
+        # A special case for the "Not Found" error, since that means that this
+        # server doesn't support adding datasets, and the user should see a more
+        # helpful response
+        if response.status_code == 404:
+            raise ValueError("Server does not support dynamically adding datasets")
+
+        if not ok(response):
+            raise ValueError("Bad Response: %s" % reason(response))
+
 
 @dispatch(Client)
 def discover(c):
@@ -143,26 +186,51 @@ def mimetype(serial):
 
 
 @dispatch(Expr, Client)
-def compute_down(expr, ec, compute_kwargs=None, odo_kwargs=None, **kwargs):
-    from .server import to_tree
-    tree = to_tree(expr)
+def compute_down(expr, ec, profiler_output=None, **kwargs):
+    """Compute down for blaze clients.
 
+    Parameters
+    ----------
+    expr : Expr
+        The expression to send to the server.
+    ec : Client
+        The blaze client to compute against.
+    namespace : dict[Symbol -> any], optional
+        The namespace to compute the expression in. This will be amended to
+        include that data for the server. By default this will just be the
+        client mapping to the server's data.
+    compute_kwargs : dict, optional
+        Extra kwargs to pass to compute on the server.
+    odo_kwargs : dict, optional
+        Extra kwargs to pass to odo on the server.
+    profile : bool, optional
+        Should blaze server run cProfile over the computation of the expression
+        and the serialization of the response.
+    profiler_output : file-like object, optional
+        A file like object to hold the profiling output from the server.
+        If this is not passed then the server will write the data to the
+        server's filesystem
+    """
+    from .server import to_tree
+
+    tree = to_tree(expr)
     serial = ec.serial
+    if profiler_output is not None:
+        kwargs['profiler_output'] = ':response'
     r = post(
         ec,
         '/compute',
-        data=serial.dumps({
-            'expr': tree,
-            'compute_kwargs': compute_kwargs,
-            'odo_kwargs': odo_kwargs,
-        }),
+        data=serial.dumps(assoc(kwargs, 'expr', tree)),
         auth=ec.auth,
         headers=mimetype(serial),
     )
 
     if not ok(r):
         raise ValueError("Bad response: %s" % reason(r))
-    return serial.loads(content(r))['data']
+    response = serial.loads(content(r))
+    if profiler_output is not None:
+        profiler_output.write(response['profiler_output'])
+    return serial.data_loads(response['data'])
 
 
 @resource.register('blaze://.+')
@@ -171,7 +239,7 @@ def resource_blaze(uri, leaf=None, **kwargs):
         raise ValueError('The syntax blaze://...::{leaf} is no longer '
                          'supported as of version 0.8.1.\n'
                          'You can access {leaf!r} using this syntax:\n'
-                         'Data({uri})[{leaf!r}]'
+                         'data({uri})[{leaf!r}]'
                          .format(leaf=leaf, uri=uri))
     uri = uri[len('blaze://'):]
     sp = uri.split('/')
